Type: Package
Package: tidygraph
Title: A Tidy API for Graph Manipulation
Version: 1.3.1
Authors@R: 
    person("Thomas Lin", "Pedersen", , "thomasp85@gmail.com", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0002-5147-4711"))
Maintainer: Thomas Lin Pedersen <thomasp85@gmail.com>
Description: A graph, while not "tidy" in itself, can be thought of as two
    tidy data frames describing node and edge data respectively.
    'tidygraph' provides an approach to manipulate these two virtual data
    frames using the API defined in the 'dplyr' package, as well as
    provides tidy interfaces to a lot of common graph algorithms.
License: MIT + file LICENSE
URL: https://tidygraph.data-imaginist.com,
        https://github.com/thomasp85/tidygraph
BugReports: https://github.com/thomasp85/tidygraph/issues
Imports: cli, dplyr (>= 0.8.5), igraph (>= 2.0.0), lifecycle, magrittr,
        pillar, R6, rlang, stats, tibble, tidyr, tools, utils
Suggests: ape, covr, data.tree, graph, influenceR, methods, netrankr,
        NetSwan, network, seriation, testthat (>= 3.0.0)
LinkingTo: cpp11
Encoding: UTF-8
RoxygenNote: 7.3.1
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2024-01-30 12:17:51 UTC; thomas
Author: Thomas Lin Pedersen [cre, aut]
    (<https://orcid.org/0000-0002-5147-4711>)
Repository: CRAN
Date/Publication: 2024-01-30 13:40:02 UTC
