# Generated by cpp11: do not edit by hand

get_paths <- function(parent) {
  .Call(`_tidygraph_get_paths`, parent)
}

collect_offspring <- function(offspring, order) {
  .Call(`_tidygraph_collect_offspring`, offspring, order)
}
